"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = renderToMarkup;

var _fastLoops = require("fast-loops");

var _felaUtils = require("fela-utils");

var _createStyleTagMarkup = _interopRequireDefault(require("./createStyleTagMarkup"));

var _getRehydrationIndex = _interopRequireDefault(require("./getRehydrationIndex"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function renderToMarkup(renderer) {
  var cacheCluster = (0, _felaUtils.clusterCache)(renderer.cache, renderer.ruleOrder);
  var rehydrationIndex = (0, _getRehydrationIndex["default"])(renderer);
  var styleMarkup = (0, _fastLoops.objectReduce)(_felaUtils.sheetMap, function (markup, type, key) {
    if (cacheCluster[key].length > 0) {
      markup += (0, _createStyleTagMarkup["default"])(cacheCluster[key], type, '', rehydrationIndex, false, renderer.styleNodeAttributes);
    }

    return markup;
  }, '');
  var support = (0, _felaUtils.cssifySupportRules)(cacheCluster.supportRules);

  if (support) {
    styleMarkup += (0, _createStyleTagMarkup["default"])(support, _felaUtils.RULE_TYPE, '', rehydrationIndex, true, renderer.styleNodeAttributes);
  }

  var mediaKeys = Object.keys(_objectSpread(_objectSpread({}, cacheCluster.supportMediaRules), cacheCluster.mediaRules)).sort(renderer.sortMediaQuery);
  return (0, _fastLoops.arrayReduce)(mediaKeys, function (markup, media) {
    // basic media query rules
    if (cacheCluster.mediaRules[media] && cacheCluster.mediaRules[media].length > 0) {
      markup += (0, _createStyleTagMarkup["default"])(cacheCluster.mediaRules[media], _felaUtils.RULE_TYPE, media, rehydrationIndex, false, renderer.styleNodeAttributes);
    } // support media rules


    if (cacheCluster.supportMediaRules[media]) {
      var mediaSupport = (0, _felaUtils.cssifySupportRules)(cacheCluster.supportMediaRules[media]);

      if (mediaSupport.length > 0) {
        markup += (0, _createStyleTagMarkup["default"])(mediaSupport, _felaUtils.RULE_TYPE, media, rehydrationIndex, true, renderer.styleNodeAttributes);
      }
    }

    return markup;
  }, styleMarkup);
}